@@ -1,3 +1,40 @@
+#' Rejection Sampling
+#'
+#' Performs rejection sampling to generate samples from a target distribution.
+#'
+#' @param n Integer. The number of samples to generate.
+#' @param f Function. The target density function from which to sample.
+#' @param max_f Numeric. The maximum value of the target density function `f`.
+#' @param proposal_fun Function. A function that generates samples from the proposal distribution.
+#' @param ... Additional arguments to be passed to the target density function `f`.
+#'
+#' @return A numeric vector of length `n` containing samples from the target distribution.
+#' @export
+#' @keywords distribution
+#'
+#' @examples
+#' target_density <- function(x) brms::dvon_mises(x, mu = 0, kappa = 10)
+#' proposal <- function(n) runif(n, min = -pi, max = pi)
+#' samples <- rejection_sampling(10000, target_density, max_f = target_density(0), proposal)
+#' hist(samples, freq = FALSE)
+#' curve(target_density, col = "red", add = TRUE)
+rejection_sampling <- function(n, f, max_f, proposal_fun, ...) {
+  stopifnot(is.numeric(n), length(n) == 1, n > 0)
+  stopifnot(is.numeric(max_f), length(max_f) == 1 | length(max_f) == n, max_f > 0)
+
+  inner <- function(n, f, max_f, proposal_fun, ..., acc = c()) {
+    if (length(acc) > n) {
+      return(acc[seq_len(n)])
+    }
+    x <- proposal_fun(n)
+    y <- stats::runif(n) * max_f
+    accept <- y < f(x, ...)
+    inner(n, f, max_f, proposal_fun, ..., acc = c(acc, x[accept]))
+  }
+
+  inner(n, f, max_f, proposal_fun, ...)
+}
+
 #' @title Distribution functions for the Signal Discrimination Model (SDM)
 #'
 #' @description Density, distribution function, and random generation for the
@@ -57,50 +94,55 @@
 #'
 #' @examples
 #' # plot the density of the SDM distribution
-#' x <- seq(-pi,pi,length.out=10000)
-#' plot(x,dsdm(x,0,2,3),type="l", xlim=c(-pi,pi),ylim=c(0,1),
-#'      xlab="Angle error (radians)",
-#'      ylab="density",
-#'      main="SDM density")
-#' lines(x,dsdm(x,0,9,1),col="red")
-#' lines(x,dsdm(x,0,2,8),col="green")
-#' legend("topright",c("c=2, kappa=3.0, mu=0",
-#'                     "c=9, kappa=1.0, mu=0",
-#'                     "c=2, kappa=8, mu=1"),
-#'        col=c("black","red","green"),lty=1, cex=0.8)
+#' x <- seq(-pi, pi, length.out = 10000)
+#' plot(x, dsdm(x, 0, 2, 3),
+#'   type = "l", xlim = c(-pi, pi), ylim = c(0, 1),
+#'   xlab = "Angle error (radians)",
+#'   ylab = "density",
+#'   main = "SDM density"
+#' )
+#' lines(x, dsdm(x, 0, 9, 1), col = "red")
+#' lines(x, dsdm(x, 0, 2, 8), col = "green")
+#' legend("topright", c(
+#'   "c=2, kappa=3.0, mu=0",
+#'   "c=9, kappa=1.0, mu=0",
+#'   "c=2, kappa=8, mu=1"
+#' ),
+#' col = c("black", "red", "green"), lty = 1, cex = 0.8
+#' )
 #'
 #' # plot the cumulative distribution function of the SDM distribution
 #' p <- psdm(x, mu = 0, c = 3.1, kappa = 5)
-#' plot(x,p,type="l")
+#' plot(x, p, type = "l")
 #'
 #' # generate random deviates from the SDM distribution and overlay the density
 #' r <- rsdm(10000, mu = 0, c = 3.1, kappa = 5)
 #' d <- dsdm(x, mu = 0, c = 3.1, kappa = 5)
-#' hist(r, breaks=60, freq=FALSE)
-#' lines(x,d,type="l", col="red")
+#' hist(r, breaks = 60, freq = FALSE)
+#' lines(x, d, type = "l", col = "red")
 dsdm <- function(x, mu = 0, c = 3, kappa = 3.5, log = FALSE,
                  parametrization = "sqrtexp") {
   stopif(isTRUE(any(kappa < 0)), "kappa must be non-negative")
   stopif(isTRUE(any(c < 0)), "c must be non-negative")
 
   .dsdm_numer <- switch(parametrization,
-                        "bessel" = .dsdm_numer_bessel,
-                        "sqrtexp" = .dsdm_numer_sqrtexp)
+    bessel = .dsdm_numer_bessel,
+    sqrtexp = .dsdm_numer_sqrtexp,
+    stop("Parametrization must be one of 'bessel' or 'sqrtexp'")
+  )
 
   lnumerator <- .dsdm_numer(x, mu, c, kappa, log = TRUE)
 
-  if (any(length(mu) > 1, length(c) > 1, length(kappa) > 1)) {
-    denom <- .dsdm_integrate_numer_v(.dsdm_numer, mu, c, kappa, lower = mu,
-                                     upper = mu + pi)
+  denom <- if (any(length(mu) > 1, length(c) > 1, length(kappa) > 1)) {
+    .dsdm_integrate_numer_v(.dsdm_numer, mu, c, kappa, lower = mu, upper = mu + pi)
   } else {
-    denom <- .dsdm_integrate_numer(.dsdm_numer, mu, c, kappa, lower = mu,
-                                   upper = mu + pi)
+    .dsdm_integrate_numer(.dsdm_numer, mu, c, kappa, lower = mu, upper = mu + pi)
   }
 
-  denom <- 2*denom
+  denom <- 2 * denom
 
   if (!log) {
-    return(exp(lnumerator)/denom)
+    return(exp(lnumerator) / denom)
   }
   lnumerator - log(denom)
 }
@@ -120,18 +162,24 @@ psdm <- function(q, mu = 0, c = 3, kappa = 3.5, lower.tail = TRUE, log.p = FALSE
   lower.bound <- (lower.bound + pi) %% pi2
 
   .dsdm_integrate <- function(mu, c, kappa, lower, upper, parametrization) {
-    stats::integrate(dsdm, lower = lower, upper = upper, mu, c, kappa,
-                     parametrization = parametrization)$value
+    stats::integrate(dsdm,
+      lower = lower, upper = upper, mu, c, kappa,
+      parametrization = parametrization
+    )$value
   }
 
   .dsdm_integrate_v <- Vectorize(.dsdm_integrate)
 
   if (any(length(q) > 1, length(mu) > 1, length(c) > 1, length(kappa) > 1)) {
-    out <- .dsdm_integrate_v(mu, c, kappa, lower = lower.bound, upper = q,
-                             parametrization = parametrization)
+    out <- .dsdm_integrate_v(mu, c, kappa,
+      lower = lower.bound, upper = q,
+      parametrization = parametrization
+    )
   } else {
-    out <-  .dsdm_integrate(mu, c, kappa, lower = lower.bound, upper = q,
-                            parametrization = parametrization)
+    out <- .dsdm_integrate(mu, c, kappa,
+      lower = lower.bound, upper = q,
+      parametrization = parametrization
+    )
   }
 
   if (!lower.tail) {
@@ -145,7 +193,7 @@ psdm <- function(q, mu = 0, c = 3, kappa = 3.5, lower.tail = TRUE, log.p = FALSE
 
 #' @rdname SDMdist
 #' @export
-qsdm <- function(p, mu=0, c=3, kappa=3.5, parametrization = "sqrtexp") {
+qsdm <- function(p, mu = 0, c = 3, kappa = 3.5, parametrization = "sqrtexp") {
   .NotYetImplemented()
 }
 
@@ -156,30 +204,22 @@ rsdm <- function(n, mu = 0, c = 3, kappa = 3.5, parametrization = "sqrtexp") {
   stopif(isTRUE(any(c < 0)), "c must be non-negative")
   stopif(length(n) > 1, "n must be a single integer")
 
-  maxy <- dsdm(0, 0, c, kappa)
-  xa <- c()
+  .dsdm_numer <- switch(parametrization,
+    bessel = .dsdm_numer_bessel,
+    sqrtexp = .dsdm_numer_sqrtexp,
+    stop("Parametrization must be one of 'bessel' or 'sqrtexp'")
+  )
 
-  .rsdm_inner <- function(n, mu, c, kappa, parametrization, xa) {
-    x <- stats::runif(n, -pi, pi)
-    y <- stats::runif(n, 0, 1) * maxy
-    accept <- y < dsdm(x, mu, c, kappa, parametrization = parametrization)
-    xa <- c(xa, x[accept])
-
-    if (length(xa) < n) {
-      return(.rsdm_inner(n, mu, c, kappa, parametrization, xa))
-    }
-
-    xa[1:n]
-  }
-
-  .rsdm_inner(n, mu, c, kappa, parametrization, xa)
+  rejection_sampling(
+    n = n,
+    f = function(x) .dsdm_numer(x, mu, c, kappa),
+    max_f = .dsdm_numer(0, 0, c, kappa),
+    proposal_fun = function(n) stats::runif(n, -pi, pi)
+  )
 }
 
 # helper functions for calculating the density of the SDM distribution
 .dsdm_numer_bessel <- function(x, mu, c, kappa, log = FALSE) {
-  stopif(isTRUE(any(kappa < 0)), "kappa must be non-negative")
-  stopif(isTRUE(any(c < 0)), "c must be non-negative")
-
   be <- besselI(kappa, nu = 0, expon.scaled = TRUE)
   out <- c * exp(kappa * (cos(x - mu) - 1)) / (2 * pi * be)
   if (!log) {
@@ -189,9 +229,6 @@ rsdm <- function(n, mu = 0, c = 3, kappa = 3.5, parametrization = "sqrtexp") {
 }
 
 .dsdm_numer_sqrtexp <- function(x, mu, c, kappa, log = FALSE) {
-  stopif(isTRUE(any(kappa < 0)), "kappa must be non-negative")
-  stopif(isTRUE(any(c < 0)), "c must be non-negative")
-
   out <- c * exp(kappa * (cos(x - mu) - 1)) * sqrt(kappa) / sqrt(2 * pi)
   if (!log) {
     out <- exp(out)
@@ -199,15 +236,13 @@ rsdm <- function(n, mu = 0, c = 3, kappa = 3.5, parametrization = "sqrtexp") {
   out
 }
 
-
-
 .dsdm_integrate_numer <- function(fun, mu, c, kappa, lower, upper) {
   stats::integrate(fun, lower = lower, upper = upper, mu, c, kappa)$value
 }
 
 .dsdm_integrate_numer_v <- Vectorize(.dsdm_integrate_numer,
-    vectorize.args = c("mu", "c", "kappa", 'lower','upper'))
-
+  vectorize.args = c("mu", "c", "kappa", "lower", "upper")
+)
 
 
 #' @title Distribution functions for the two-parameter mixture model (mixture2p)
@@ -244,20 +279,20 @@ rsdm <- function(n, mu = 0, c = 3, kappa = 3.5, parametrization = "sqrtexp") {
 #' @examples
 #' # generate random samples from the mixture2p model and overlay the density
 #' r <- rmixture2p(10000, mu = 0, kappa = 4, p_mem = 0.8)
-#' x <- seq(-pi,pi,length.out=10000)
+#' x <- seq(-pi, pi, length.out = 10000)
 #' d <- dmixture2p(x, mu = 0, kappa = 4, p_mem = 0.8)
-#' hist(r, breaks=60, freq=FALSE)
-#' lines(x,d,type="l", col="red")
+#' hist(r, breaks = 60, freq = FALSE)
+#' lines(x, d, type = "l", col = "red")
 #'
-dmixture2p <- function(x, mu=0, kappa=5, p_mem = 0.6, log = FALSE) {
+dmixture2p <- function(x, mu = 0, kappa = 5, p_mem = 0.6, log = FALSE) {
   stopif(isTRUE(any(kappa < 0)), "kappa must be non-negative")
   stopif(isTRUE(any(p_mem < 0)), "p_mem must be larger than zero.")
   stopif(isTRUE(any(p_mem > 1)), "p_mem must be smaller than one.")
 
   density <- matrix(data = NaN, nrow = length(x), ncol = 2)
 
-  density[,1] <- log(p_mem) + brms::dvon_mises(x = x,mu = mu , kappa = kappa, log = T)
-  density[,2] <- log(1 - p_mem) + brms::dvon_mises(x = x,mu = 0 , kappa = 0, log = T)
+  density[, 1] <- log(p_mem) + brms::dvon_mises(x = x, mu = mu, kappa = kappa, log = T)
+  density[, 2] <- log(1 - p_mem) + brms::dvon_mises(x = x, mu = 0, kappa = 0, log = T)
 
   density <- matrixStats::rowLogSumExps(density)
 
@@ -270,43 +305,31 @@ dmixture2p <- function(x, mu=0, kappa=5, p_mem = 0.6, log = FALSE) {
 
 #' @rdname mixture2p_dist
 #' @export
-pmixture2p <- function(q, mu=0, kappa=7, p_mem = 0.8) {
+pmixture2p <- function(q, mu = 0, kappa = 7, p_mem = 0.8) {
   .NotYetImplemented()
 }
 
 #' @rdname mixture2p_dist
 #' @export
-qmixture2p <- function(p, mu=0, kappa=5, p_mem = 0.6) {
+qmixture2p <- function(p, mu = 0, kappa = 5, p_mem = 0.6) {
   .NotYetImplemented()
 }
 
 #' @rdname mixture2p_dist
 #' @export
-rmixture2p <- function(n, mu=0, kappa=5, p_mem = 0.6) {
+rmixture2p <- function(n, mu = 0, kappa = 5, p_mem = 0.6) {
   stopif(isTRUE(any(kappa < 0)), "kappa must be non-negative")
   stopif(isTRUE(any(p_mem < 0)), "p_mem must be larger than zero.")
   stopif(isTRUE(any(p_mem > 1)), "p_mem must be smaller than one.")
 
-  maxy <- dmixture2p(0, 0, kappa, p_mem)
-  xa <- c()
-
-  .rmixture2p_inner <- function(n, mu, c, kappa, p_mem, xa) {
-    x <- stats::runif(n, -pi, pi)
-    y <- stats::runif(n, 0, 1) * maxy
-    accept <- y < dmixture2p(x, mu, kappa, p_mem)
-    xa <- c(xa, x[accept])
-
-    if (length(xa) < n) {
-      return(.rmixture2p_inner(n, mu, c, kappa, p_mem, xa))
-    }
-
-    xa[1:n]
-  }
-
-  .rmixture2p_inner(n, mu, c, kappa, p_mem, xa)
+  rejection_sampling(
+    n = n,
+    f = function(x) dmixture2p(x, mu, kappa, p_mem),
+    max_f = dmixture2p(0, 0, kappa, p_mem),
+    proposal_fun = function(n) stats::runif(n, -pi, pi)
+  )
 }
 
-
 #' @title Distribution functions for the three-parameter mixture model (mixture3p)
 #'
 #' @description Density, distribution, and random generation functions for the
@@ -345,29 +368,31 @@ rmixture2p <- function(n, mu=0, kappa=5, p_mem = 0.6) {
 #' @examples
 #' # generate random samples from the mixture3p model and overlay the density
 #' r <- rmixture3p(10000, mu = c(0, 2, -1.5), kappa = 4, p_mem = 0.6, p_nt = 0.2)
-#' x <- seq(-pi,pi,length.out=10000)
+#' x <- seq(-pi, pi, length.out = 10000)
 #' d <- dmixture3p(x, mu = c(0, 2, -1.5), kappa = 4, p_mem = 0.6, p_nt = 0.2)
-#' hist(r, breaks=60, freq=FALSE)
-#' lines(x,d,type="l", col="red")
+#' hist(r, breaks = 60, freq = FALSE)
+#' lines(x, d, type = "l", col = "red")
 #'
-dmixture3p <- function(x, mu=c(0,2,-1.5), kappa = 5, p_mem = 0.6, p_nt = 0.2, log = FALSE) {
+dmixture3p <- function(x, mu = c(0, 2, -1.5), kappa = 5, p_mem = 0.6, p_nt = 0.2, log = FALSE) {
   stopif(isTRUE(any(kappa < 0)), "kappa must be non-negative")
   stopif(isTRUE(any(p_mem < 0)), "p_mem must be larger than zero.")
   stopif(isTRUE(any(p_nt < 0)), "p_nt must be larger than zero.")
   stopif(isTRUE(any(p_mem + p_nt > 1)), "The sum of p_mem and p_nt must be smaller than one.")
 
   density <- matrix(data = NaN, nrow = length(x), ncol = length(mu) + 1)
-  probs <- c(p_mem,
-             rep(p_nt/(length(mu) - 1), each = length(mu) - 1),
-             (1 - p_mem - p_nt))
+  probs <- c(
+    p_mem,
+    rep(p_nt / (length(mu) - 1), each = length(mu) - 1),
+    (1 - p_mem - p_nt)
+  )
 
   for (i in 1:(length(mu))) {
-    density[,i] <- log(probs[i]) +
+    density[, i] <- log(probs[i]) +
       brms::dvon_mises(x = x, mu = mu[i], kappa = kappa, log = T)
   }
 
-  density[,length(mu) + 1] <- log(probs[length(mu) + 1]) +
-    stats::dunif(x = x,-pi, pi, log = T)
+  density[, length(mu) + 1] <- log(probs[length(mu) + 1]) +
+    stats::dunif(x = x, -pi, pi, log = T)
 
   density <- matrixStats::rowLogSumExps(density)
 
@@ -380,42 +405,33 @@ dmixture3p <- function(x, mu=c(0,2,-1.5), kappa = 5, p_mem = 0.6, p_nt = 0.2, lo
 
 #' @rdname mixture3p_dist
 #' @export
-pmixture3p <- function(q, mu=c(0,2,-1.5), kappa = 5, p_mem = 0.6, p_nt = 0.2) {
+pmixture3p <- function(q, mu = c(0, 2, -1.5), kappa = 5, p_mem = 0.6, p_nt = 0.2) {
   .NotYetImplemented()
 }
 
 #' @rdname mixture3p_dist
 #' @export
-qmixture3p <- function(p, mu=c(0,2,-1.5), kappa = 5, p_mem = 0.6, p_nt = 0.2) {
+qmixture3p <- function(p, mu = c(0, 2, -1.5), kappa = 5, p_mem = 0.6, p_nt = 0.2) {
   .NotYetImplemented()
 }
 
 #' @rdname mixture3p_dist
 #' @export
-rmixture3p <- function(n, mu=c(0,2,-1.5), kappa = 5, p_mem = 0.6, p_nt = 0.2) {
+rmixture3p <- function(n, mu = c(0, 2, -1.5), kappa = 5, p_mem = 0.6, p_nt = 0.2) {
   stopif(isTRUE(any(kappa < 0)), "kappa must be non-negative")
   stopif(isTRUE(any(p_mem < 0)), "p_mem must be larger than zero.")
   stopif(isTRUE(any(p_nt < 0)), "p_nt must be larger than zero.")
   stopif(isTRUE(any(p_mem + p_nt > 1)), "The sum of p_mem and p_nt must be smaller than one.")
 
-  x <- seq(-pi,pi,length.out = 361)
-  maxy <- max(dmixture3p(x, mu, kappa, p_mem, p_nt))
-  xa <- c()
+  xm <- seq(-pi, pi, length.out = 361)
+  max_y <- max(dmixture3p(xm, mu, kappa, p_mem, p_nt))
 
-  .rmixture3p_inner <- function(n, mu, c, kappa, p_mem, p_nt, xa) {
-    x <- stats::runif(n, -pi, pi)
-    y <- stats::runif(n, 0, 1) * maxy
-    accept <- y < dmixture3p(x, mu, kappa, p_mem, p_nt)
-    xa <- c(xa, x[accept])
-
-    if (length(xa) < n) {
-      return(.rmixture3p_inner(n, mu, c, kappa, p_mem, p_nt, xa))
-    }
-
-    xa[1:n]
-  }
-
-  .rmixture3p_inner(n, mu, c, kappa, p_mem, p_nt, xa)
+  rejection_sampling(
+    n = n,
+    f = function(x) dmixture3p(x, mu, kappa, p_mem, p_nt),
+    max_f = max_y,
+    proposal_fun = function(n) stats::runif(n, -pi, pi)
+  )
 }
 
 #' @title Distribution functions for the Interference Measurement Model (IMM)
@@ -457,40 +473,46 @@ rmixture3p <- function(n, mu=c(0,2,-1.5), kappa = 5, p_mem = 0.6, p_nt = 0.2) {
 #'
 #' @examples
 #' # generate random samples from the imm and overlay the density
-#' r <- rimm(10000, mu = c(0, 2, -1.5), dist = c(0, 0.5, 2),
-#'           c = 5, a = 2, s = 2, b = 1, kappa = 4)
-#' x <- seq(-pi,pi,length.out=10000)
-#' d <- dimm(x, mu = c(0, 2, -1.5), dist = c(0, 0.5, 2),
-#'           c = 5, a = 2, s = 2, b = 1, kappa = 4)
-#' hist(r, breaks=60, freq=FALSE)
-#' lines(x,d,type="l", col="red")
+#' r <- rimm(10000,
+#'   mu = c(0, 2, -1.5), dist = c(0, 0.5, 2),
+#'   c = 5, a = 2, s = 2, b = 1, kappa = 4
+#' )
+#' x <- seq(-pi, pi, length.out = 10000)
+#' d <- dimm(x,
+#'   mu = c(0, 2, -1.5), dist = c(0, 0.5, 2),
+#'   c = 5, a = 2, s = 2, b = 1, kappa = 4
+#' )
+#' hist(r, breaks = 60, freq = FALSE)
+#' lines(x, d, type = "l", col = "red")
 #'
-dimm <- function(x, mu=c(0,2,-1.5), dist = c(0,0.5,2),
-                 c=5, a = 2, b = 1, s = 2, kappa=5, log = FALSE) {
+dimm <- function(x, mu = c(0, 2, -1.5), dist = c(0, 0.5, 2),
+                 c = 5, a = 2, b = 1, s = 2, kappa = 5, log = FALSE) {
   stopif(isTRUE(any(kappa < 0)), "kappa must be non-negative")
-  stopif(length(mu) != length(dist),
-         "The number of items does not match the distances provided from the cued location.")
+  len_mu <- length(mu)
+  stopif(
+    len_mu != length(dist),
+    "The number of items does not match the distances provided from the cued location."
+  )
   stopif(isTRUE(any(s < 0)), "s must be non-negative")
   stopif(isTRUE(any(dist < 0)), "all distances have to be positive.")
 
   # compute activation for all items
-  weights <- rep(c, length(mu)) * exp(-s*dist) + rep(a, length(mu))
+  weights <- rep(c, len_mu) * exp(-s * dist) + rep(a, len_mu)
 
   # add activation of background noise
-  weights <- c(weights,b)
+  weights <- c(weights, b)
 
   # compute probability for responding stemming from each distribution
-  probs <- weights/sum(weights)
+  probs <- weights / sum(weights)
+  density <- matrix(data = NaN, nrow = length(x), ncol = len_mu + 1)
 
-  density <- matrix(data = NaN, nrow = length(x), ncol = length(mu) + 1)
-
-  for (i in 1:(length(mu))) {
-    density[,i] <- log(probs[i]) +
-      brms::dvon_mises(x = x, mu = mu[i], kappa = kappa, log = T)
+  for (i in seq_along(mu)) {
+    density[, i] <- log(probs[i]) +
+      brms::dvon_mises(x, mu = mu[i], kappa = kappa, log = T)
   }
 
-  density[,length(mu) + 1] <- log(probs[length(mu) + 1]) +
-    stats::dunif(x = x,-pi, pi, log = T)
+  density[, len_mu + 1] <- log(probs[len_mu + 1]) +
+    stats::dunif(x = x, -pi, pi, log = T)
 
   density <- matrixStats::rowLogSumExps(density)
 
@@ -503,44 +525,117 @@ dimm <- function(x, mu=c(0,2,-1.5), dist = c(0,0.5,2),
 
 #' @rdname IMMdist
 #' @export
-pimm <- function(q, mu=c(0,2,-1.5), dist = c(0,0.5,2),
-                 c=1, a = 0.2, b = 0, s = 2, kappa=5) {
+pimm <- function(q, mu = c(0, 2, -1.5), dist = c(0, 0.5, 2),
+                 c = 1, a = 0.2, b = 0, s = 2, kappa = 5) {
   .NotYetImplemented()
 }
 
 #' @rdname IMMdist
 #' @export
-qimm <- function(p, mu=c(0,2,-1.5), dist = c(0,0.5,2),
-                 c=1, a = 0.2, b = 0, s = 2, kappa=5) {
+qimm <- function(p, mu = c(0, 2, -1.5), dist = c(0, 0.5, 2),
+                 c = 1, a = 0.2, b = 0, s = 2, kappa = 5) {
   .NotYetImplemented()
 }
 
 #' @rdname IMMdist
 #' @export
-rimm <- function(n, mu=c(0,2,-1.5), dist = c(0,0.5,2),
-                 c=1, a = 0.2, b = 1, s = 2, kappa=5) {
+rimm <- function(n, mu = c(0, 2, -1.5), dist = c(0, 0.5, 2),
+                 c = 1, a = 0.2, b = 1, s = 2, kappa = 5) {
   stopif(isTRUE(any(kappa < 0)), "kappa must be non-negative")
-  stopif(length(mu) != length(dist),
-         "The number of items does not match the distances provided from the cued location.")
   stopif(isTRUE(any(s < 0)), "s must be non-negative")
   stopif(isTRUE(any(dist < 0)), "all distances have to be positive.")
+  stopif(
+    length(mu) != length(dist),
+    "The number of items does not match the distances provided from the cued location."
+  )
 
-  x <-  seq(-pi,pi,length.out = 361)
-  maxy <- max(dimm(x, mu, dist, c, a, b, s, kappa))
-  xa <- c()
+  xm <- seq(-pi, pi, length.out = 361)
+  max_y <- max(dimm(xm, mu, dist, c, a, b, s, kappa))
 
-  .rimm_inner <- function(n, mu, dist, c, a, b, s, kappa, xa) {
-    x <- stats::runif(n, -pi, pi)
-    y <- stats::runif(n, 0, 1) * maxy
-    accept <- y < dimm(x, mu, dist, c, a, b, s, kappa)
-    xa <- c(xa, x[accept])
-
-    if (length(xa) < n) {
-      return(.rimm_inner(n, mu, dist, c, a, b, s, kappa, xa))
-    }
-
-    xa[1:n]
-  }
-
-  .rimm_inner(n, mu, dist, c ,a ,b ,s ,kappa , xa)
+  rejection_sampling(
+    n = n,
+    f = function(x) dimm(x, mu, dist, c, a, b, s, kappa),
+    max_f = max_y,
+    proposal_fun = function(n) stats::runif(n, -pi, pi)
+  )
 }
+
+#' @title Distribution functions for the Memory Measurement Model (M3)
+#'
+#' @description Density and random generation functions for the memory
+#'   measurement model. Please note that these functions are currently not
+#'   vectorized.
+#'
+#' @name m3dist
+#'
+#' @param x Integer vector of length `K` where K is the number of response categories 
+#'   and each value is the number of observed responses per category
+#' @param n Integer. Number of observations to generate data for
+#' @param size The total number of observations in all categories 
+#' @param pars A named vector of parameters of the memory measurement model
+#' @param m3_model A `bmmodel` object specifying the m3 model that densities or
+#'   random samples should be generated for
+#' @param act_funs A `bmmformula` object specifying the activation functions for
+#'   the different response categories for the "custom" version of the M3. The 
+#'   default will attempt to construct the standard activation functions for the 
+#'   "ss" and "cs" model version. For a custom m3 model you need to specify the
+#'   act_funs argument manually
+#' @param log Logical; if `TRUE` (default), values are returned on the log scale.
+#' @param ... can be used to pass additional variables that are used in the
+#'   activation functions, but not parameters of the model
+#'
+#' @keywords distribution
+#'
+#' @references Oberauer, K., & Lewandowsky, S. (2019). Simple measurement models
+#'   for complex working-memory tasks. Psychological Review, 126(6), 880–932.
+#'   https://doi.org/10.1037/rev0000159
+#'
+#' @return `dm3` gives the density of the memory measurement model, and `rm3`
+#'   gives the random generation function for the memory measurement model.
+#'
+#' @examples
+#'   model <- m3(
+#'    resp_cats = c("corr", "other", "npl"),
+#'    num_options = c(1, 4, 5),
+#'    choice_rule = "simple",
+#'    version = "ss"
+#'  )
+#'  dm3(x = c(20, 10, 10), pars = c(a = 1, b = 1, c = 2), m3_model = model)
+#' @export
+dm3 <- function(x, pars, m3_model, act_funs = construct_m3_act_funs(m3_model, warnings = FALSE), 
+                log = TRUE, ...) {
+  probs <- .compute_m3_probability_vector(pars, m3_model, act_funs, ...)
+  dmultinom(x, prob = probs, log = log)
+}
+
+#' @rdname m3dist
+#' @export
+rm3 <- function(n, size, pars, m3_model, act_funs = construct_m3_act_funs(m3_model, warnings = FALSE),
+                ...) {
+  probs <- .compute_m3_probability_vector(pars, m3_model, act_funs, ...)
+  t(rmultinom(n, size = size, prob = probs))
+}
+
+.compute_m3_probability_vector <-
+  function(pars, m3_model, act_funs = construct_m3_act_funs(m3_model, warnings = FALSE), ...) {
+    pars <- c(pars, unlist(list(...)))
+    stopif(
+      is_try_error(try(act_funs, silent = TRUE)),
+      'No activation functions for version "custom" provided.
+      Please pass activation functions for the different response categories
+      using the "act_funs" argument.'
+    )
+    stopif(
+      !identical(sort(rhs_vars(act_funs)), sort(names(pars))),
+      'The names or number of parameters used in the activation functions mismatch the names or number
+      of parameters ("pars") and additional arguments (i.e. ...) passed to the function.'
+    )
+
+    acts <- sapply(act_funs, function(pform) eval(pform[[length(pform)]], envir = as.list(pars)))
+
+    num_options <- m3_model$other_vars$num_options
+    choice_rule <- tolower(m3_model$other_vars$choice_rule)
+    if (choice_rule == "softmax") acts <- exp(acts)
+    acts <- acts * num_options
+    acts / sum(acts)
+  }
